const chalk = require ('chalk')
const _ = require ('lodash')
const fs = require ('fs')
const path = require ('path')
const globby = require ('globby')
const querystring = require ('querystring')

function findFile(file) {
  let stat
  try {
    stat = fs.statSync (file)
  } catch (error) {
  }
  if (stat && stat.isFile ()) {
    return file
  }
  return undefined
}

/**
 *
 * @param program
 * @param ksr
 * @param isDev
 * @returns entries = entry[]
 * interface entry {
 *   template: string //模板地址 附加`ejs-loader`路径
 *   real_path: string //模板路径
 *   basename: string //基本文件路径
 *   path: string //基本文件路径
 *   link: []string //基本文件路径
 *   filename: string //导出文件路径,相对于dist目录
 *   ext: string //文件后缀
 * }
 */
module.exports = function (entry, ksr) {
  if (!entry || entry.length <= 0) {
    return []
  }
  const pathConvert = ksr.pathConvert
  const entries = _.reduce (
    entry,
    (files, file) => {
      const [ glob, qs = '' ] = file.split (':')
      const finds = globby.sync (pathConvert (glob))
      finds.forEach ((file) => {
        files.push ({
          source: file,
          qs: qs
        })
      })
      return files
    },
    []
  )

  const cwd = process.cwd ()
  const result = _.reduce (
    entries,
    function (files, entry) {
      let source = entry.source
      let qs = entry.qs
      const _path = pathConvert (source)
      try {
        if (!findFile (_path)) {
          return
        }
        let extname = path.extname (_path)
        let ext = extname.replace (/\./, '')
        const basename = path.basename (_path, extname)
        const relative = path.relative (cwd, source)
        const relative_dir = path.dirname (relative)
        const dirname = path.dirname (_path)
        const qs_option_params = {
          dir: relative_dir,
          filename: `${dirname}.${ext}`,
          basename: dirname,
          ext: ext
        }

        let qs_option = qs.replace (/\[(.*?)\]/g, (all, metched) => {
          return qs_option_params[metched] || metched
        })
        //查询参数
        qs_option = querystring.parse (qs_option)

        //  *   template: string //模板地址 附加`ejs-loader`路径
        //  *   real_path: string //模板路径
        //  *   basename: string //基本文件路径
        //  *   path: string //输出路径
        //  *   link: []string //引用文件地址
        //  *   ext: string //文件后缀

        let qs_option_path = qs_option.path || relative.replace (/\/|\\/g, '_')
        let qs_option_link = qs_option.link ? qs_option.link.split (';') : []
        const obj = {
          path: qs_option_path,
          real_path: _path,
          vfs: `${qs_option_path}_entry.js`,
          link: qs_option_link,
          ext: ext,
          template: undefined
        }

        if (ext !== 'js' && ext !== 'ts' &&
          ext !== 'html' && ext !== 'ejs'
        ) {
          return files
        }

        if (ext === 'html' || ext === 'ejs') {
          obj.template = `!!ejs-loader!${_path}`
          const js = path.resolve (dirname, `${basename}.js`)
          if (findFile (js)) {
            obj.link.push (js)
          }
          const ts = path.resolve (dirname, `${basename}.ts`)
          if (findFile (ts)) {
            obj.link.push (ts)
          }
        }
        files.push (obj)
      } catch (error) {
        console.error (`自动解析入口失败${chalk.red (error)}`)
        process.exit (1)
      }
      return files
    },
    []
  )
  console.log(result)
  return result
}
