const webpack = require('webpack');
const chalk = require('chalk');
const webpackConfig = require('../configuration/webpack/webpackConfig');
module.exports = function (program) {
    const cfg = webpackConfig(program, 'build');
    const compiler = webpack(cfg.webpack);
    compiler.run((err, stats) => {
        const options = {
            assets: true,
            modules: false,
            children: false,
            cached: false,
            errors: true,
            warnings: true,
            errorDetails: true,
            chunks: false, // 使构建过程更静默无输出
            colors: true // 在控制台展示颜色
        };
        if (err) {
            console.error(err.stack || err);
            if (err.details) {
                console.error(err.details);
            }
        }
        if (stats.hasErrors() || stats.hasWarnings()) {
            options['modules'] = true;
            options['children'] = true;
            options['chunks'] = true;
        }
        console.log(stats.toString(options));
    });
};